"use client"

import { CodeNode } from "@lexical/code"
import { LinkNode } from "@lexical/link"
import { ListItemNode, ListNode } from "@lexical/list"
import {
  InitialConfigType,
  LexicalComposer,
} from "@lexical/react/LexicalComposer"
import { ContentEditable } from "@lexical/react/LexicalContentEditable"
import { LexicalErrorBoundary } from "@lexical/react/LexicalErrorBoundary"
import { HistoryPlugin } from "@lexical/react/LexicalHistoryPlugin"
import { HorizontalRuleNode } from "@lexical/react/LexicalHorizontalRuleNode"
import { LinkPlugin } from "@lexical/react/LexicalLinkPlugin"
import { ListPlugin } from "@lexical/react/LexicalListPlugin"
import { MarkdownShortcutPlugin } from "@lexical/react/LexicalMarkdownShortcutPlugin"
import { OnChangePlugin } from "@lexical/react/LexicalOnChangePlugin"
import { RichTextPlugin } from "@lexical/react/LexicalRichTextPlugin"
import { HeadingNode, QuoteNode } from "@lexical/rich-text"
import { EditorState } from "lexical"
import { useCallback } from "react"
import ToolbarPlugin from "./plugins/ToolbarPlugin"

type Props = {
  onChange?: (editorState: EditorState) => void
  placeholder?: string
  initialEditorState?: string
}

export default function LexicalEditor({
  onChange,
  placeholder = "Write your Terms of Service here...",
  initialEditorState,
}: Props) {
  const initialConfig: InitialConfigType = {
    namespace: "TosEditor",
    editorState: initialEditorState ?? undefined,
    theme: {
      paragraph: "mb-2",
      heading: {
        h1: "text-2xl font-bold",
        h2: "text-xl font-semibold",
        h3: "text-lg font-medium",
      },
      quote: "border-l-4 pl-4 italic text-muted-foreground",
      list: {
        nested: {
          listitem: "ml-6",
        },
        ol: "list-decimal list-inside",
        ul: "list-disc list-inside",
        listitem: "my-1",
      },
      code: "bg-muted text-sm font-mono rounded p-1",
    },
    nodes: [
      HeadingNode,
      QuoteNode,
      CodeNode,
      ListNode,
      ListItemNode,
      HorizontalRuleNode,
      LinkNode,
    ],
    onError: (error) => {
      console.error("Lexical Error:", error)
    },
  }

  const handleChange = useCallback(
    (editorState: EditorState) => {
      if (onChange) onChange(editorState)
    },
    [onChange]
  )

  return (
    <LexicalComposer initialConfig={initialConfig}>
      <div className="border rounded-md shadow-sm bg-background text-foreground p-4">
        <ToolbarPlugin />
        <div className="relative min-h-[200px] mt-2">
          <RichTextPlugin
            contentEditable={
              <ContentEditable className="min-h-[200px] border rounded-md p-4 bg-background text-foreground focus:outline-none" />
            }
            placeholder={
              <div className="absolute top-2 left-2 text-muted-foreground pointer-events-none text-sm">
                {placeholder}
              </div>
            }
            ErrorBoundary={LexicalErrorBoundary}
          />
        </div>
        <HistoryPlugin />
        <ListPlugin />
        <LinkPlugin />
        <MarkdownShortcutPlugin />
        <OnChangePlugin onChange={handleChange} />
      </div>
    </LexicalComposer>
  )
}
