"use client";

import { defaultValueCtx, Editor, rootCtx } from "@milkdown/core";
import { block } from "@milkdown/plugin-block";
import { clipboard } from "@milkdown/plugin-clipboard";
import { cursor } from "@milkdown/plugin-cursor";
import { history } from "@milkdown/plugin-history";
import { indent } from "@milkdown/plugin-indent";
import { listener, listenerCtx } from "@milkdown/plugin-listener";
import { prism } from "@milkdown/plugin-prism";
import { gfm } from "@milkdown/preset-gfm";
import { Milkdown, useEditor } from "@milkdown/react";
import { useState } from "react";

import { Button } from "@/components/ui/button";
import { Tabs, TabsContent, TabsList, TabsTrigger } from "@/components/ui/tabs";
import { Textarea } from "@/components/ui/textarea";

interface MilkdownEditorProps {
  initialMarkdown?: string;
  onSave?: (markdown: string, html: string) => void;
}

export function MilkdownEditor({ initialMarkdown = "", onSave }: MilkdownEditorProps) {
  const [markdown, setMarkdown] = useState(initialMarkdown);
  const [html, setHtml] = useState("");
  const [mode, setMode] = useState<"editor" | "markdown">("editor");

  useEditor((root) => {
    return Editor.make()
      .config((ctx) => {
        ctx.set(rootCtx, root);
        ctx.set(defaultValueCtx, initialMarkdown);

        const listener = ctx.get(listenerCtx);
        listener.updated((_ctx, doc) => {
          const md = doc?.toString() ?? "";
          setMarkdown(md);

          const htmlContent = document.querySelector(".milkdown")?.innerHTML ?? "";
          setHtml(htmlContent);
        });
      })
      .use(gfm)
      .use(listener)
      .use(history)
      .use(clipboard)
      .use(cursor)
      .use(indent)
      .use(prism)
      .use(block);
  });

  const handleSave = () => {
    onSave?.(markdown, html);
  };

  return (
    <div className="w-full space-y-4">
      <Tabs value={mode} onValueChange={(value) => setMode(value as "editor" | "markdown")}>
        <TabsList className="w-full justify-start">
          <TabsTrigger value="editor">WYSIWYG</TabsTrigger>
          <TabsTrigger value="markdown">Markdown</TabsTrigger>
        </TabsList>

        <TabsContent value="editor">
          <div className="border rounded-lg p-4 bg-background">
            <Milkdown />
          </div>
        </TabsContent>

        <TabsContent value="markdown">
          <Textarea
            value={markdown}
            onChange={(e) => setMarkdown(e.target.value)}
            rows={20}
            className="font-mono"
          />
        </TabsContent>
      </Tabs>

      <div className="flex justify-end">
        <Button onClick={handleSave}>Save</Button>
      </div>
    </div>
  );
}
